/*
 *  Copyright (C) 2015  Thadeu Lima de Souza Cascardo <cascardo@cascardo.eti.br>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "isento.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include "cmd.h"
#include "list.h"
#include "util.h"
#include "totais.h"

static int isento_totais_update(struct declaracao *dec, struct isento *isento)
{
	int r = 0;
	switch (isento->codigo) {
	case 82:
		isento->exclusivo = 0;
		r = totais_add(dec, "DOACOES", isento->valor);
		break;
	case 93:
		isento->exclusivo = 0;
		r = totais_add(dec, "INDENIZACOES", isento->valor);
		break;
	case 96:
		isento->exclusivo = 1;
		r = totais_add(dec, "PLR", isento->valor);
		break;
	case 97:
		break;
	case 98:
		isento->exclusivo = 0;
		r = totais_add(dec, "POUPANCA", isento->valor);
		break;
	case 99:
		isento->exclusivo = 1;
		r = totais_add(dec, "APLICACOES", isento->valor);
		break;
	}
	if (isento->exclusivo) {
		r += totais_add(dec, "EXCLUSIVOS", isento->valor);
		if (isento->dependente) {
			r += totais_add(dec, "EXCLUSIVOSDEP", isento->valor);
		} else {
			r += totais_add(dec, "EXCLUSIVOSTIT", isento->valor);
		}
	} else {
		r += totais_add(dec, "ISENTOS", isento->valor);
		if (isento->dependente) {
			r += totais_add(dec, "ISENTOSDEP", isento->valor);
		} else {
			r += totais_add(dec, "ISENTOSTIT", isento->valor);
		}
	}
	return r;
}

void isento_free(void *pointer)
{
	struct isento *isento = pointer;
	if (isento->cnpj)
		free(isento->cnpj);
	if (isento->nome)
		free(isento->nome);
	if (isento->descricao)
		free(isento->descricao);
	free(isento);
}

static int isento_cmp(void *p1, void *p2)
{
	struct isento *r1 = p1;
	struct isento *r2 = p2;
	if (r1->exclusivo != r2->exclusivo)
		return r2->exclusivo - r1->exclusivo;
	/* O rendimento maior vem primeiro. */
	if (r1->valor > r2->valor)
		return -1;
	else if (r1->valor < r2->valor)
		return 1;
	return 0;
}

static struct isento * isento_new(char **args, int argc)
{
	struct isento *isento;
	int r = 0;
	isento = malloc(sizeof(*isento));

	if (!strcmp(args[0], "exclusivo")) {
		isento->exclusivo = 1;
	}

	r += set_int(args[1], &isento->codigo);
	r += set_string(args[2], &isento->cnpj);
	r += set_string(args[3], &isento->nome);
	r += set_llong(args[4], &isento->valor);
	if (r < 0 || isento->codigo < 0 ||
	    isento->valor < 0) {
		isento_free(isento);
		return NULL;
	}
	if (argc == 6) {
		r = set_int(args[5], &isento->dependente);
	} else {
		isento->dependente = 0;
	}
	if (r < 0 || isento->dependente < 0) {
		isento_free(isento);
		return NULL;
	}
	if (argc == 7) {
		r = set_string(args[6], &isento->descricao);
		if (r < 0) {
			isento_free(isento);
			return NULL;
		}
	} else {
		isento->descricao = NULL;
	}
	return isento;
}

static int run_isento(struct declaracao *dec, char **args, int argc)
{
	struct isento *isento;
	int r;
	if (argc < 5 || argc > 7)
		return -EINVAL;
	isento = isento_new(args, argc);
	if (!isento)
		return -ENOMEM;
	if (isento->dependente > list_size(dec->dependentes)) {
		isento_free(isento);
		return -EINVAL;
	}
	r = isento_totais_update(dec, isento);
	if (r) {
		isento_free(isento);
		return r;
	}
	r = list_insert_ordered(&dec->isentos, isento, isento_cmp);
	if (r < 0) {
		isento_free(isento);
		return r;
	}
	return 0;
}

void isento_salva(struct declaracao *dec, FILE *f)
{
	int i;
	struct isento *j;
	for (i = 0; j = list_get(dec->isentos, i); i++) {
		if (j->codigo == 97 && j->exclusivo)
			fprintf(f, "exclusivo ");
		else
			fprintf(f, "isento ");
		fprintf(f, "%d \"%s\" \"%s\" %lld %d",
			j->codigo, j->cnpj, j->nome, j->valor, j->dependente);
		if (j->descricao)
			fprintf(f, " \"%s\"", j->descricao);
		fprintf(f, "\n");
	}
}

static struct cmd cmd_isento = {
	.name = "isento",
	.run = run_isento,
};

static struct cmd cmd_exclusivo = {
	.name = "exclusivo",
	.run = run_isento,
};

int isento_cmd_init(void)
{
	cmd_add(&cmd_isento);
	cmd_add(&cmd_exclusivo);
	return 0;
}

struct isento * isento_get(struct declaracao *dec, int codigo, int n)
{
	struct isento *isento;
	int i;
	int j = 0;
	for (i = 0; (isento = list_get(dec->isentos, i)); i++) {
		if (isento->codigo == codigo && j++ == n)
			break;
	}
	return isento;
}
